{smcl}
{* *! version 1.0.0  22Fe2009}{...}
{cmd:help gbsarms} 
{hline}

{title:Title}

{p2colset 5 16 18 2}{...}
{p2col :{hi:gbsarms} {hline 2}}Adaptive rejection Metropolis sampling for use in Gibbs sampling{p_end}
{p2colreset}{...}


{title:Syntax}

{phang2}
{cmd:gbsarms} {it:logpost rowvector parameter}
{ifin}
{weight}{cmd:,}
{opth g:rid(numlist)}
[{it:options}]

{phang}
{it:logpost} must be the name of the user's program for calculating the log of
the conditional posterior density.
See {help mhsnorm##userprog:Structure for the user's program} in
{helpb mhsnorm} for more details on how to write the program for the log
posterior.

{phang}
{it:rowvector} must be the name of the row vector containing the parameter
values.

{phang}
{it:parameter} must be the number of the parameter that is being updated.

{synoptset 16 tabbed}{...}
{synopthdr}
{synoptline}
{p2coldent:* {opth g:rid(numlist)}}initial grid of points{p_end}
{synopt:{opt l:bound(#)}}lower limit of range{p_end}
{synopt:{opt u:bound(#)}}upper limit of range{p_end}
{synopt:{opt max:try(#)}}maximum number of attempts to generate a new value{p_end}
{synopt:{opt logp(#)}}log posterior at the current point{p_end}
{synoptline}
{p2colreset}{...}
{p 4 6 2}
* {cmd:grid()} is required.{p_end}
{p 4 6 2}
{opt fweight}s, {opt aweight}s, {opt iweight}s, and {opt pweight}s are
allowed; see {help weight}.


{title:Description}

{pstd}
{cmd:gbsarms} uses adaptive rejection Metropolis sampling (ARMS) to
simulate a value from a one-dimensional posterior distribution. It is useful
in Gibbs sampling. The grid consists of an initial set of at least 3 points
placed across the region where the posterior is thought to lie. Values can be
generated outside the range of this initial grid but not beyond the upper and
lower bounds.  The log posterior is evaluated at each point in the grid, and
those values are used to define an envelope enclosing the log posterior. The
algorithm starts by using the envelope for adaptive rejection sampling (ARS)
in which rejected points are added to the grid. To allow for the possibility
that the envelope might not always lie above the log posterior, the user
must subject the point obtained from rejection sampling to a final
Metropolis-Hastings step.  Because a Metropolis step is involved in generating
the new value, it is not valid to place the initial grid dependent on previous
values from the same chain.  The user must supply a program for evaluating the
log posterior for any given set of parameter values; see {helpb mhsnorm} for
details of the structure of the program. The user's program may be written in
Stata or Mata, but the calculations for {cmd:gbsarms} use Mata for speed.
The Mata code is stored in the library {cmd:libmcmc}, which must be installed
for the command to work.

{pstd}
{helpb gbsars} is similar to ARMS but does not have the Metropolis step.
{cmd:gbsars} is slightly more efficient but only works if the log posterior is
log concave.


{title:Options}

{phang}
{opth grid(numlist)} specifies a set of at least 3 ascending values defining
the initial grid for the envelope that encloses the log posterior.
{cmd:grid()} is required.{p_end}

{phang}
{opt lbound(#)} specifies the lower limit of the range of the posterior.
The default is minus infinity.{p_end}

{phang}
{opt ubound(#)} specifies the upper limit of the range of the posterior.
The default is infinity.{p_end}

{phang}
{opt maxtry(#)} specifies the number of attempts before giving up.
The default is {cmd:maxtry(50)}. If the program takes more than a handful of
attempts, then either the posterior is extremely awkward or the initial grid
was very poorly placed.{p_end}

{phang}
{opt logp(#)} specifies the log posterior for the current set of parameter
values. If {cmd:logp()} is set to missing, this value is calculated by the
program.{p_end}


{title:Stored results}

{pstd}
{cmd:gbsarms} stores the following in {cmd:r()}:

{synoptset 12 tabbed}{...}
{p2col 5 12 16 2: Scalars}{p_end}
{synopt:{cmd:r(logp)}}log posterior at the new point{p_end}
{synopt:{cmd:r(try)}}number of evaluations of the log posterior{p_end}
{p2colreset}{...}


{title:Example}

{phang}{cmd:. gbsarms mylogpost b 3, grid(3 4 5 10) lbound(0)}{p_end}


{title:Author}

{pstd} John Thompson{break}
       Department of Health Sciences{break}
       University of Leicester, UK
       
{pstd}
Please report any errors to
{browse "mailto:john.thompson@le.ac.uk":john.thompson@le.ac.uk}.
{p_end}
