*============================================================================
* gbsars:  ARS sampling for a single parameter
* Author: John Thompson
* Date:   Dec 2012
*============================================================================
program gbsars, rclass
   version 12.1
   syntax anything [fweight pweight aweight iweight] [if] [in], ///
   Grid(numlist ascending min = 3) ///  user specified grid of points
   [                               ///
   Lbound(real -1e20)              ///  lower bound for parameter
   Ubound(real 1e20)               ///  upper bound for parameter
   MAXtry(integer 50)              ///  maximum attempts to sample before giving up
   ]
   marksample touse
   
   tokenize `"`anything'"'
   local logfunct "`1'"
   local b "`2'"
   local ipar "`3'"
   if "`weight'" != "" local wt "[`weight'`exp']"
   
   local logp = "."
   tempname slp
*-------------------------------
* Check grid specification
*-------------------------------   
   local n : word count `grid'
   tokenize `grid'
   if `lbound' >= `1' | `ubound' <= `n' {
      di as err "Bounds not consistent with grid"
      exit(499)
   }
   if `lbound' < -1e19 local lbound "."
   if `ubound' >  1e19 local ubound "."
*-------------------------------
* Call Mata for calculation
*-------------------------------   
   mata: gbsars_stata("`logfunct' `slp' `b' `ipar' `wt' if `touse'","`slp'",`logp',"`b'",`ipar',"`grid'",`lbound',`ubound',`maxtry')
   return scalar try = `try'
   return scalar logp = `logp'
   return scalar fail = `fail'
end

