*============================================================================
* gbsslice:  Slice sampling for a single parameter
* Author: John Thompson
* Date:   Dec 2012
*============================================================================
program gbsslice, rclass
   version 12.1
   syntax anything [fweight pweight aweight iweight] [if] [in], [ ///
   Stepsize(real 1)     ///   stepsize 
   Nstep(integer 6)     ///   maximum number of steps
   LOGP(real 1e20)      ///   value of log-posterior at current point (if known)
   Lbound(real -1e20)   ///   lower bound on the parameter
   Ubound(real 1e20)    ///   upper bound on the parameter
   MAXtry(integer 50)   ///   maximum number of sampled points before giving up
   ]
   marksample touse
   
   tokenize "`anything'"
   local logfunct "`1'"
   local b "`2'"
   local ipar "`3'"
   if "`weight'" != "" local wt "[`weight'`exp']"
   
   if `lbound' < -1e19 local lbound "."
   else if `b'[1,`ipar'] < `lbound' {
      di as err "Parameter `ipar': initial value outside the bounds"
	  exit(499)
   }
   if `ubound' > 1e19 local ubound "."
   else if `b'[1,`ipar'] > `ubound' {
      di as err "Parameter `ipar': initial value outside the bounds"
	  exit(499)
   }
   if `logp'   > 1e19 local logp = "."
   tempname slp
   mata: gbsslice_stata("`logfunct' `slp' `b' `ipar' `wt' if `touse'","`slp'",`logp',"`b'",`ipar',`stepsize',`nstep',`lbound',`ubound',`maxtry')
   return scalar try1 = `try1'
   return scalar try2 = `try2'
   return scalar fail = `fail'
   return scalar logp = `logp'
end

