*============================================================================
* mcmcac:  Autocorrelation plots - wrapper for Stata's ac and pac commands
* Author: John Thompson
* Date:   Dec 2012
*============================================================================

program mcmcac
   version 12.1
   syntax varlist [if] [in] , [ ///
   PAC                        ///  pac instead of ac
   GOPTions(string asis)      ///  options for plotting individual graphs
   CGOPTions(string asis)     ///  options for combining graphs
   ACOPTions(string asis)     ///  options sent to ac
   Save(string)               ///  .dta file to save plotting points
   ]
   
   quietly {
      marksample touse , novarlist

*-----------------------------
* Count number of plots
*-----------------------------
      local np : word count `varlist'
      if `np' > 1 local nodraw "nodraw"      
*-----------------------------
* parse the save option
*-----------------------------
      tokenize `"`save'"',parse(",")
      local save = trim(subinstr(`"`1'"',`"""',"",.))
      local dreplace `"`3'"'
*-----------------------------
* save any existing tsset
*-----------------------------
      cap tsset
      local oldt = r(timevar)
*-----------------------------
* create each plot
*-----------------------------      
      tempvar t plot corr lag
      gen `t' = _n
      tsset `t'
      local plots ""
      local i = 0
      foreach v of varlist `varlist' {
         local i = `i' + 1
         tempfile t`i'
         local plots "`plots' `t`i''.gph"
*-----------------------------
* call ac or pac
*-----------------------------         
         if "`pac'" != "" {
            pac `v' if `touse' , `acoptions' `goptions' saving("`t`i''.gph",replace) gen(`corr') `nodraw'
         }
         else {
            ac `v' if `touse' , `acoptions' `goptions' saving("`t`i''.gph",replace) gen(`corr') `nodraw'
         }
*-----------------------------
* if needed save the points
*-----------------------------
         if "`save'" != "" {
            preserve
            keep `corr'
            gen `plot' = `i'
            gen `lag' = _n
            if `i' > 1 {
               append using "`save'"
               save "`save'" , replace
            }
            else {
               save "`save'" , `dreplace'
            }
            restore
         }
         drop `corr'
      }
*-----------------------------
* combine plots if needed
*-----------------------------
      if `np' > 1 {
         graph combine `plots' , `cgoptions'
      }
*-----------------------------
* tidy the saved points
*-----------------------------      
      if "`save'" != "" {
         preserve
         use "`save'", clear
         rename `plot' plot
         rename `corr' r
         rename `lag' lag
         drop if r == .
         sort plot lag
         save "`save'", replace
         restore
      }
*-----------------------------
* restore the old tsset
*-----------------------------
      cap tsset `oldt'
   }
end
