{smcl}
{* *! version 1.0.0  dec2012}{...}
{cmd:help mcmccheck} 
{hline}

{title:Title}

{p2colset 5 18 20 2}{...}
{p2col :{hi:mcmccheck} {hline 2}}Graphical Bayesian residual checking{p_end}
{p2colreset}{...}


{title:Syntax}

{phang2}
{cmd:mcmccheck}{cmd:,}
{opt p:redictions(string)}
{opt pf:ilename(string)}
[{it:options}]

{synoptset 25 tabbed}{...}
{synopthdr}
{synoptline}
{p2coldent:* {opt p:redictions(string)}}variables containing the predictions{p_end}
{p2coldent:* {opt pf:ilename(string)}}file containing the predictions{p_end}
{synopt:{opt d:ata(string)}}observed data value or variable{p_end}
{synopt:{opt df:ilename(string)}}file containing the observed data{p_end}
{synopt:{opt pl:ot(string)}}type of plot{p_end}
{synopt:{opt x:axis(string)}}variable or formula for x axis{p_end}
{synopt:{opt y:axis(string)}}variable or formula for y axis{p_end}
{synopt:{opt gopt:ions(string)}}any {helpb twoway_options:twoway} options to
affect rendition of the individual plots{p_end}
{synopt:{opt cgopt:ions(string)}}any {helpb graph combine} options to affect
rendition of combining plots{p_end}
{synopt:{opt kopt:ions(string)}}smoothing options passed directly to {helpb kdensity}{p_end}
{synopt:{cmdab:s:ave(}{it:string}{cmd:, replace)}}{cmd:.dta} results file to save the residuals{p_end}
{synopt:{opt by(string)}}variable to create different symbols on a scatterplot{p_end}
{synopt:{opt lc:ondition(string)}}condition to control whether points are labeled{p_end}
{synopt:{opt lv:ariable(string)}}variable used to label points{p_end}
{synopt:{opt ns:amples(#)}}number of samples for {cmd:plot(epp)} {p_end}
{synoptline}
{p2colreset}{...}
{p 4 6 2}
* {cmd:predictions()} and {cmd:pfilename()} are required.


{title:Description}

{pstd}
{cmd:mcmccheck} performs graphical Bayesian model checking based on residuals.
The program calculates the mean of the predictive distribution ({cmd:fit}) and
five basic measures of surprise or unusualness of an observation:{p_end}

{phang}1. residual ({cmd:residual}): difference of a value from its mean
    prediction divided by the standard deviation of its predictive
    distribution{p_end}
{phang}2. absolute residual ({cmd:absresidual}): the absolute of the residual
    in item 1{p_end}
{phang}3. posterior predictive p-value ({cmd:ppp}): tail area under the
     predictive distribution that is greater than the observation{p_end}
{phang}4. two-tail posterior predictive p-value ({cmd:tppp}): tail area under
     the predictive distribution corresponding to values more extreme than the
     observation{p_end}
{phang}5. relative predictive surprise ({cmd:rps}): height of the predictive
     density at the observed value as a percentage of the maximum height of
     the density{p_end}


{title:Options}

{phang}
{opt predictions(string)} specifies the single predicted value or prefix for
variables for a set of predicted values to be found in the file given by the
{cmd:pfilename()} option. For instance, {cmd:predictions(pred_)} would imply
the set of variables {cmd:pred_1}, {cmd:pred_2}, etc.  {cmd:predictions()}
is required.

{phang}
{opt pfilename(string)} specifies the file containing the predictions.
{cmd:pfilename()} is required.

{phang}
{opt data(string)} specifies the actual data variable name or a numerical
value.

{phang}
{opt dfilename(string)} specifies the file containing the data.

{phang}
{opt plot(string)} specifies the type of plot. {it:string} must be one of
the following:

{p2colset 12 24 26 2}{...}
{synopt:{cmd:scatter}}scatterplot{p_end}
{synopt:{cmd:lowess}}scatterplot with lowess smoother{p_end}
{synopt:{cmd:epp}}empirical probability plot{p_end}
{synopt:{cmd:histogram}}histogram of the residuals{p_end}
{synopt:{cmd:density}}smoothed density estimate{p_end}
{synopt:{cmd:boxplot}}boxplot of the residuals{p_end}
{synopt:{cmd:summary}}combined plot showing scatterplots, empirical
probability plots, and histograms{p_end}
{synopt:{cmd:none}}used to save measures of surprise without plotting{p_end}

{pmore}
If the {cmd:data()} option contains a single value, then the default plottype
is {cmd:histogram}. If the {cmd:data()} option contains a variable, then the
default plottype is {cmd:scatter}.{p_end}

{phang}
{opt xaxis(string)} specifies the variable name or formula for the x axis of a
plot. {it:string} may be one of the following: {cmd:residual}, {cmd:ppp},
{cmd:rps}, or {cmd:fit} (mean prediction).

{phang}
{opt yaxis(string)} specifies the variable name or formula for the y axis of a
plot. {it:string} may be one of the following: {cmd:residual}, {cmd:ppp},
{cmd:rps}, or {cmd:fit} (mean prediction).

{phang}
{opt goptions(string)} are any of the {helpb twoway_options:twoway} options
for controlling individual plots. 

{phang}
{opt cgoptions(string)} are any of the {helpb graph combine} options to
control the combining of the plots in plottype summary. 

{phang}
{opt koptions(string)} are any of the options passed to {helpb kdensity} for
plottypes {cmd:density} and {cmd:distribution}.{p_end}

{phang}
{cmd:save(}{it:string}{cmd:, replace)} specifies the Stata data file
({cmd:.dta}) for saving the measures of surprise
calculated for any twoway plots or by plottype {cmd:none}.

{phang}
{opt by(string)} specifies the categorical variable. With scatterplots,
different symbols are used for each level of the categorical variable.{p_end}

{phang}
{opt lcondition(string)} specifies that when users label points on a
scatterplot using {cmd:lvariable()}, only those points that satisfy this
condition are labeled.{p_end}

{phang}
{opt lvariable(string)} specifies the variable used to label the points on a
scatterplot.{p_end}

{phang}
{opt nsamples(#)} specifies the number of samples taken when constructing an
empirical probability plot.{p_end}


{title:Examples}

{pstd}Create a summary plot for observed data {cmd:y} with corresponding
predictions stored as {cmd:pred_y_1}, {cmd:pred_y_2}, etc.{p_end}
{phang2}{cmd:. mcmccheck, predictions(pred_y_) pfile(mcmc.dta) data(y) dfile(data.dta) plot(summary)}

{pstd}Plot a histogram of the predictions corresponding to observation
{cmd:y[4]}, naming the plot{p_end}
{phang2}{cmd:. mcmccheck, predictions(pred_y_4) pfile(mcmc.dta) data(y[4]) dfile(data.dta) goptions(name(plot1))}

{pstd}Draw a scatterplot of {cmd:rps} against the log of the fitted values
{p_end}
{phang2}{cmd:. mcmccheck, predictions(pred_y_) pfile(mcmc.dta) data(y) dfile(data.dta) plot(scatter) xaxes(log(fit)) yaxes(rps)}

            
{title:Author}

{pstd}
John Thompson{break}
Department of Health Sciences{break}
University of Leicester, UK{break}

{pstd}Please report any errors to
{browse "mailto:john.thompson@le.ac.uk":john.thompson@le.ac.uk}.
{p_end}
