*============================================================================
* mcmcmahal:  plots of the mahalanobis distance of set of points from their mean
* Author:  John Thompson
* Date:    Dec 2012
*============================================================================
program mcmcmahal
   version 12.1
   syntax varlist [if] [in] [ , ///
       Distance(name)           ///
       NOPlot                   ///
       REPLACE                  ///
       GOPTions(string)        ///
       CGOPTions(string)        ///
   ]

   marksample touse
   tempname C M MH V
*---------------------------------
* find mean and variance
*---------------------------------
   qui matrix accum `C' = `varlist' `if' `in', dev nocon means(`M')
   qui count `if' `in'
   local n = r(N)
   matrix `C' = syminv(`C'/`n')
*---------------------------------
* Add variable to contain distances
*---------------------------------
   if "`distance'" == "" {
      tempvar distance
   }
   else {
      if "`replace'" != "" cap drop `distance'
   }
   qui gen `distance' = .
   label var `distance' "Mahalanobis Distance"
*---------------------------------
* Calculate distance
*---------------------------------
   local m = _N
   forvalues i = 1/`m' {
      if `touse'[`i'] == 1 {
         local values ""
         local comma ""
         foreach v of varlist `varlist' {
            local a = `v'[`i']
            local values "`values'`comma'`a'"
            local comma ","
         }
         matrix `V' = (`values')
         matrix `MH' = (`V'-`M')*`C'*(`V'-`M')'
         qui replace `distance' = `MH'[1,1] in `i'
      }
   }
*---------------------------------
* create the plots
*---------------------------------
   if "`noplot'" == "" {
      local p = 0
      foreach v of varlist `varlist' {
         local ++p
      }
      tempfile plot1 plot2 plot3 plot4
      mcmctrace `distance' `if' `in', gopt(`goptions' ytitle(Mahalanobis Distance) nodraw )
      qui graph save `plot1' , replace
      mcmccusum `distance' `if' `in', ref(3) gopt(`goptions' ytitle(CUSUM) nodraw)
      qui graph save `plot2' , replace
      qchi `distance', df(`p') nodraw title(" ")  `goptions' 
      qui graph save `plot3' , replace
      mcmcintervals `distance' `if' `in', m(50) gopt(`goptions' title(" ") ytitle(80% interval for Distance) nodraw)
      qui graph save `plot4' , replace
      graph combine "`plot1'" "`plot2'" "`plot3'" "`plot4'" , `cgoptions'
   }
end

