{smcl}
{cmd:help mcmcrun}
{hline}

{title:Title}

{p2colset 5 16 18 2}{...}
{p2col :{hi:mcmcrun} {hline 2}}Housekeeping program for the generation of complete MCMC runs{p_end}
{p2colreset}{...}


{title:Syntax}

{phang2}
{cmd:mcmcrun} {it:logpost rowvector}
{cmd:using} {it:{help filename}}
{ifin}
{weight}{cmd:,}
{opt s:amplers(string)}
[{it:options}]

{phang}
{it:logpost} must be the name of the user's program for calculating the log of
the conditional posterior density.
See {help mhsnorm##userprog:Structure for the user's program} in
{helpb mhsnorm} for more details on how to write the program for the log
posterior.

{phang}
{it:rowvector} must be the name of the row vector containing the parameter
values.

{synoptset 20 tabbed}{...}
{synopthdr}
{synoptline}
{p2coldent:* {opt s:amplers(string)}}list of samplers{p_end}
{synopt:{opt b:urnin(#)}}length of the burn-in{p_end}
{synopt:{opt u:pdates(#)}}length of the chain{p_end}
{synopt:{opt t:hin(#)}}thinning number{p_end}
{synopt:{opt adapt}}adapt MH samplers during the burn-in{p_end}
{synopt:{opt pa:rameters(string)}}names of the parameters{p_end}
{synopt:{opt pr:edictions(string)}}name of a function that calculates the predictions{p_end}
{synopt:{opt rep:lace}}overwrite the output file if it exists{p_end}
{synopt:{opt app:end}}append to the output file{p_end}
{synopt:{opt jp:osterior}}user's program calculates joint log posterior{p_end}
{synopt:{opt savelogp:}}add log posterior to the output file{p_end}
{synopt:{opt dot:s(#)}}iterations between dots that monitor progress{p_end}
{synopt:{opt mata:}}use Mata rather than Stata{p_end}
{synopt:{opt d:ata(string)}}data matrices when using Mata{p_end}
{synoptline}
{p2colreset}{...}
{p 4 6 2}
* {cmd:samplers()} is required.{p_end}
{p 4 6 2}
{opt fweight}s are allowed; see {help weight}.


{title:Description}

{pstd}
{cmd:mcmcrun} controls the calculation of MCMC simulations in Stata or Mata.
It is simply a housekeeping program that calls samplers specified by the user
and writes the updated parameter values to the comma-delimited ({cmd:.csv})
text file as specified by {cmd:using}.  Following the command name, the user
must provide the name of the program that calculates the log posterior and the
name of the row vector that contains the parameter values set to their initial
values. When Mata is used, the list must also include the name of the data
matrix that is passed to the user's program for calculating the log posterior.
Thus the list consists of the name of the user's program, the data matrix, and
the row vector of parameters.


{title:Options}

{phang}
{opt samplers(string)} specifies a sampler for each parameter.  Each chosen
sampler is placed in parentheses that must contain a keyword to denote the
choice followed by the options needed to run that sampler. The MH samplers can
be referred to either by their full names, for example, {bf:mhslogn}, or
without the leading "mhs", as in {bf:logn}.  In a similar way, a Gibbs sampler
such as {bf:gbsslice} may be referred to by its full name or as {bf:slice}.
MCMC recognizes the four MH samplers {cmd:logn}, {cmd:norm}, {cmd:mnorm},
and {cmd:trnc} and the four Gibbs samplers {cmd:arms}, {cmd:ars},
{cmd:griddy}, and {cmd:slice}.  {bf:skip} (without any parameters) implies
that this parameter is not to be updated.  It will therefore be left at its
initial value throughout the run.  Any other keyword is assumed to refer to a
program written by the user, and in this way, the user can add his or her own
samplers. If the parentheses containing a sampler is preceded by a number,
then this means that the same sampler is to be used repeatedly; for instance,
{cmd:samplers(3(norm,} ...{cmd:))} means that the next three parameters will
be updated using the sampler {cmd:mhsnorm}. Any options required by the
sampler must also be included in the parentheses.  Because the MH samplers
require a standard deviation for the proposal distribution, this might be
specified as {cmd:samplers(mhslogn, sd(2))}.{p_end}

{phang}
{opt burnin(#)} specifies the length of the burn-in. The burn-in is not saved
in the output file. The default is {cmd:burnin(0)}.

{phang}
{opt updates(#)} specifies the length of the chain. The default is
{cmd:updates(1000)}.

{phang}
{opt thin(#)} specifies the thinning number. The default is {cmd:thin(1)},
meaning no thinning.

{phang}
{opt adapt} specifies to adapt MH samplers during the burn-in to tune the
proposal standard deviation.

{phang}
{opt parameters(string)} specifies the names of the parameters separated by
spaces for use in the output file.  Dashes are allowed to denote a run of
parameter names; for example, {cmd:parameters(a1-a5)} is equivalent to
{cmd:parameters(a1 a2 a3 a4 a5)}.

{phang}
{opt predictions(string)} specifies the name of a function that calculates the
predictions.  Predictions are added to the output file under the same name as
the function.  Thus, if the function creating the predictions is called
{cmd:ys}, then the predictions will be added to the output file under the
names {cmd:ys1}, {cmd:ys2}, ....  In Stata, the user-written function must
accept a single argument equal to the current parameter estimates and return
the string {cmd:r(pred)}, which contains the name of a Stata matrix containing
the predictions as a row vector.  In Mata, the user-written function must accept
two arguments equal to the data matrix and the row vector of parameter
estimates; it must return the row vector of predictions directly.

{phang}
{opt replace} specifies that the comma-delimited output file be overwritten
if it already exists.

{phang}
{opt append} specifies to append the results to the output file.

{phang}
{opt jposterior} specifies that {it:logpost} calculates the full log posterior
rather than the log-conditional posterior. In this case, there is no need to
recalculate the log posterior at the current set of parameter values; rather,
the last calculated value can be used.

{phang}
{opt savelogp} specifies to add the final log posterior calculated after a
complete cycle through the parameters to the output file.

{phang}
{opt dots(#)} specifies the number of iterations between the
progress-monitoring dots that are displayed in the Results window. The default
is {cmd:dots(250)}. Specifying zero or a negative value will stop any dots
from being shown. In Mata, the dots can slow down the program noticeably on
some computers.

{phang}
{opt mata} declares that the user's programs are written in Mata rather than
Stata.

{phang}
{opt data(string)} specifies the list of Stata matrices or scalars separated
by spaces. This option is only required when using Mata. These scalars and
matrices are copied from Stata to Mata under the same names. Use
{cmd:X=(x1)} to save a single Stata variable as a Mata matrix.
{cmd:X=(x1 x2 x3)} can be used to copy three Stata variables {cmd:x1},
{cmd:x2}, and {cmd:x3} to Mata and save them as a Mata matrix X. 


{title:Writing your own samplers}

{pstd}
A sampler with an unrecognized name, that is, not {cmd:skip} or one of the
"mhs" or "gbs" samplers, will be assumed to be a user-written sampler.
This might refer to the user's Gibbs sampler for updating a standard
distribution, or it might be that the user has written his or her own general
purpose sampler.  The user's program is called, followed by the name of the
program for calculating the log posterior, the name of the row vector
containing the parameter values, and ipar, the number of the first parameter to
be updated.  The {cmd:if} and {cmd:in} conditions and any options are passed
directly to the user's program with the exception of the {opt dim(#)} option,
which is interpreted as giving the dimension of the update, for example, the
number of consecutive parameters updated by the user's program. If {cmd:dim()}
is omitted, it is assumed that the program updates a single parameter.
{opt dim()} is not passed to the user's program.  For samplers written in
Mata, the arguments are in order: the current value of the log posterior, a
pointer to the function that calculates the log posterior, the data matrix,
the row vector of parameter values, and a scalar giving the number of the
parameters to be updated. These arguments are followed by any other options in
the order in which they are specified by the calling program.  Whether you use
Stata or Mata, you will need to include the argument for the log-posterior
function or a dummy replacement even if it is not used.{p_end}


{title:Example}

{phang}{cmd:. matrix s = (0.5, 2) }{p_end}
{phang}{cmd:. mcmcrun logpost b using temp.csv, samplers(2(norm, sd(s)))}{break}
       {cmd:parameters(a1 a2) burn(1000) adapt update(10000)}{p_end}


{title:Author}

{pstd}
John Thompson{break}
Department of Health Sciences{break}
University of Leicester, UK{break}

{pstd}
Please report any errors to
{browse "mailto:john.thompson@le.ac.uk":john.thompson@le.ac.uk}.
{p_end}
