*============================================================================
* mhslogn: Metropolis-Hastings Sampler based on a log-normal proposal distribution
* Author: John Thompson
* Date:   Dec 2012
*============================================================================

program mhslogn, rclass
   version 12.1
   syntax anything  [fweight] [if] [in], ///
   SD(string)        ///
   [                 ///
   LOGP(real 1e20)   /// 
   DEBUG             ///
   ]
   
   tokenize "`anything'"
   local logpost "`1'"
   local b       "`2'"
   local ipar   = `3'
   
   tempname logl newlogl
   if "`debug'" == "" local qui "qui"
   
   local oldvalue = `b'[1,`ipar']
   if `oldvalue' <= 0 {
      di as err "Parameter `ipar': initial value not positive"
	  exit(499)
   }
   scalar `logl' = .
   if `logp' < 1e19 scalar `logl' = `logp'
   if `logl' == .  {
      `qui' `logpost' `logl' `b' `ipar' [`weight'`exp'] `if' `in'
      if `logl' == .  {
         di as err "mhslogn: cannot calculate log-posterior for parameter `ipar'"
         matrix list `b'
         exit(1400)
      }
   }
   local newvalue = `oldvalue'*exp( `sd'*rnormal() )
   matrix `b'[1,`ipar'] = `newvalue'
   
   scalar `newlogl' = .
   `qui' `logpost' `newlogl' `b' `ipar' [`weight'`exp']  `if' `in'
   if `newlogl' == .  {
      di as err "mhslogn: cannot calculate log-posterior for parameter `ipar'"
      matrix list `b'
      exit(1400)
   }
   
   local logalpha = (`newlogl'+ log(`newvalue')) - (`logl' + log(`oldvalue'))
   if log(uniform()) < `logalpha'   local accept = 1
   else {
      local accept = 0
      matrix `b'[1,`ipar'] = `oldvalue'
      scalar `newlogl' = `logl'
   }
   return scalar accept = `accept'
   return scalar logp = `newlogl'
end
