{smcl}
{* *! version 1.0.0 dec2012}{...}
{cmd:help mhstrnc} 
{hline}

{title:Title}

{p2colset 5 16 18 2}{...}
{p2col :{hi:mhstrnc} {hline 2}}Metropolis-Hastings algorithm based on a random walk with a Gaussian proposal on a logistically transformed scale{p_end}
{p2colreset}{...}


{title:Syntax}

{phang2}
{cmd:mhstrnc} {it:logpost rowvector parameter} {ifin} {weight}{cmd:,}
{opt sd(#)}
[{it:options}]

{phang}
{it:logpost} must be the name of the user's program for calculating the log of
the conditional posterior density.
See {help mhsnorm##userprog:Structure for the user's program} in
{helpb mhsnorm} for more details on how to write the program for the log
posterior.

{phang}
{it:rowvector} must be the name of the rowvector containing the parameter
values.

{phang}
{it:parameter} must be the number of the parameter that is being updated.

{synoptset 14 tabbed}{...}
{synopthdr}
{synoptline}
{p2coldent:* {opt sd:(#)}}standard deviation of the proposal distribution{p_end}
{synopt:{opt l:bound(#)}}lower bound of the parameter's range{p_end}
{synopt:{opt u:bound(#)}}upper bound of the parameter's range{p_end}
{synopt:{opt logp(#)}}value of log posterior at current point{p_end}
{synoptline}
{p2colreset}{...}
{p 4 6 2}
* {cmd:sd()} is required.{p_end}
{p 4 6 2}
{opt fweight}s are allowed; see {help weight}.


{title:Description}

{pstd}
{cmd:mhstrnc} uses the Metropolis-Hastings algorithm to create a single update
of a single parameter.  Because it uses a logistic transform, it is suitable
for parameters that can only take values over a bounded range, for instance, a
proportion defined over the range [0,1]. If the range of the parameter is
[a,b], then the normal proposal is generated on the transformed scale
log[(y-a)/(b-y)].  Repeated calls to {cmd:mhstrnc} can be used to create an
MCMC algorithm and hence estimate the posterior distribution of the parameter.
The user must supply a program that evaluates the log posterior; this program
must have the structure described in {helpb mhsnorm}. Parameters are passed in
a row vector.  The efficiency but not the validity of the algorithm is affected
by the choice of standard deviation for the proposal.  Values between one-half
and three-quarters of the size of the range of the posterior (on the
transformed scale) usually work well. It is not valid to use previous results
from the same chain to alter the standard deviation of the proposal
distribution.{p_end}


{title:Options}

{phang}
{opt sd(#)} specifies the standard deviation of the proposal distribution as
chosen by the user. This {it:#} affects the efficiency of the algorithm.
Values that are too small will lead to small changes in the parameter and thus
to slow convergence. Values that are too large produce proposales that
are frequently rejected, again causing slow convergence.{p_end}

{phang}
{opt lbound(#)} specifies the lower bound of the parameter's range. To make
the bound equal to minus infinity, set it to a missing value.{p_end}

{phang}
{opt ubound(#)} specifies the upper bound of the parameter's range. To make
the bound infinite, set it to a missing value.  Setting {cmd:lbound(.)} and
{cmd:ubound(.)} is equivalent to calling {helpb mhsnorm}, and setting
{cmd:lbound(0)} and {cmd:ubound(.)} is equivalent to calling {helpb mhslogn}.
{p_end}

{phang}
{opt logp(#)} specifies the value of the log posterior at the current point.
If given and not missing, then this value is used, and the value is not
recalculated, which saves one call to the user's program {it:logpost}.{p_end}


{title:Stored results}

{pstd}
{cmd:mhstrnc} stores the following in {cmd:r()}:

{p2colset 9 20 22 2}{...}
{p2col 5 12 16 2: Scalars}{p_end}
{synopt:{cmd:r(logp)}}log posterior for the updated set of parameters{p_end}
{synopt:{cmd:r(accept)}}{cmd:1} if the proposal was accepted and {cmd:0} otherwise{p_end}
{p2colreset}{...}


{title:Example}

{phang}{cmd:. mhstrnc logpost b 3, sd(0.1) lb(0) ub(1)}{p_end}


{title:Comment}

{pstd}
The transform used is log[(y-lbound)/(ubound-y)], where the parameter's range
is (lbound,ubound).  When parameters can take any real value, use
{helpb mhsnorm}, and for parameters that must be positive, use
{helpb mhslogn}.  {helpb mhsmnorm} updates a set of parameters using a
multivariate Gaussian distribution.{p_end}


{title:Author}

{pstd}
John Thompson{break}
Department of Health Sciences{break}
University of Leicester, UK{break}

{pstd}
Please report any errors to
{browse "mailto:john.thompson@le.ac.uk":john.thompson@le.ac.uk}.
{p_end}
