{smcl}
{* *! version 1.0.0  22May2006}{...}
{cmd:help wbslist} 
{hline}

{title:Title}

{p2colset 5 16 18 2}{...}
{p2col :{hi:wbslist} {hline 2}}Writes mixed data as a WinBUGS list{p_end}
{p2colreset}{...}


{title:Syntax}

{p 8 16 2}
{cmd:wbslist}
{cmd:(}{it:component1} {ifin} [{cmd:,} {it:suboptions}]{cmd:)}{break}
{cmd:(}{it:component2} {ifin} [{cmd:,} {it:suboptions}]{cmd:)}{break}
{cmd:(}...{cmd:)}
[{cmd:using} {it:{help filename}}]
[{cmd:,} {cmd:replace}]

{pstd}
The contents of each parentheses may begin with one of the following keywords:
{cmd:scalar}, {cmd:vector}
({cmd:variable} is a synonym for {cmd:vector}),
{cmdab:mat:rix}, {cmdab:struc:ture}, or {cmd:table}.
A structure is a matrix within WinBUGS that is created from a set of Stata
variables.  Tables are multiway structures created by specifying a variable
containing the table contents and then one variable for each dimension
containing values 1, 2, .... 
If the contents of the parentheses do not begin with any of these keywords,
then the contents are treated as text and are written directly to the output
file. A call to {cmd:wbslist} might be{p_end}

{phang2}{cmd:. wbslist (matrix R S) (struc y1 y2 y3, name(Y)) using data.txt, replace}

{pstd}
where {cmd:R} and {cmd:S} are Stata matrices, and {cmd:y1}, {cmd:y2}, and
{cmd:y3} are Stata variables.{p_end}

{synoptset 16}{...}
{synopt:{it:suboptions}}Description{p_end}
{synoptline}
{synopt:{opt f:ormats(string)}}output formats{p_end}
{synopt:{opt l:inesize(#)}}number of values per line in the output file{p_end}
{synopt:{opt n:ames(string)}}alternative names for the variables{p_end}
{synoptline}
{p2colreset}{...}

{synoptset 16}{...}
{synopt:{it:option}}Description{p_end}
{synoptline}
{synopt:{opt replace}}overwrite the output file if it exists{p_end}
{synoptline}
{p2colreset}{...}


{title:Description}

{pstd}
{cmd:wbslist} writes data or initial values to a text file in an R-like list
structure suitable for reading into WinBUGS. The data may be any combination
of Stata scalars, variables, matrices, two-way structures created from Stata
variables, multiway tables, or values typed directly into the command by the
user. 

{pstd}
WinBUGS can also read arrays of data, but these must be created using the
{helpb wbarray} command and stored in a separate file. WinBUGS can read
multiple data files.


{title:Suboptions}

{pstd}
{cmd:scalar}, {cmd:vector}, {cmd:matrix}, and {cmd:structure} all allow three
options. These suboptions are placed after {it:component} within the
parentheses.

{phang}
{opt formats(string)} specifies the formats used for writing the data. The
default is {cmd:formats(%8.3f)}. Formats cycle so that
{cmd:formats(%4.0f %3.1f)} would write the data using
{cmd:%4.0f %3.1f %4.0f} ....

{phang}
{opt linesize(#)} specifies the number of values per line in the output file.
The default is {cmd:linesize(10)}.

{phang}
{opt names(string)} specifies alternative names to be used in the WinBUGS data
file. In this way, data may have a name in WinBUGS that is different from
the name used in Stata. If this option is used, the number of alternative names
must match the number of items.

{pstd}
{cmd:vector} and {cmd:structure} also allow {cmd:if} and {cmd:in} to limit the
data that are written to the list. These are also placed within the
parentheses.


{title:Option}

{phang}
{opt replace} specifies that the output file be overwritten if it already
exists.


{title:Writing strings}

{pstd}
When the parentheses do not start with a keyword, the contents are simply
added to the output file. However, prior to writing, two expansions are
applied based on curly brackets ({cmd:{}}) and square brackets ({cmd:[]}).
These features are primarily intended for generating initial values.  The
contents of {cmd:{}} are repeatedly evaluated prior to writing. Thus

{phang2}{cmd:. wbslist (alpha=c(5{c -(}0{c )-})) using inits.txt, replace}

{pstd}
would write a list containing {cmd:alpha=c(0,0,0,0,0)} and

{phang2}{cmd:. wbslist (alpha=c(5{c -(}rnormal(){c )-})) using inits.txt, replace}

{pstd}
would write a list containing 5 different values generated by evaluating the
Stata function {cmd:rnormal()}. Square brackets refer to the contents of a
Stata variable. Thus

{phang2}{cmd: . wbslist (alpha=c(5{c -(}a[3]{c )-})) using inits.txt, replace}

{pstd}
would write a list in which alpha had 5 values taken from rows 3, 4, 5, 6, and
7 of the Stata variable {cmd:a}.


{title:Example}

{pstd}Appropriate scalars and vectors defined{p_end}
{phang2}{cmd:. wbslist (scalar a b, f(%4.0f)) (vector x in 1/3)}{break}
        {cmd:using data.txt, replace}{p_end}


{title:Additional software}

{pstd}
The WinBUGS Manual is available from {browse www.mrc-bsu.cam.ac.uk/bugs}.
Information on OpenBUGS is available from {browse www.openbugs.net/}.
See {search winbugs} for a list of all the commands for running WinBUGS from
Stata.


{title:Author}

{pstd}
John Thompson{break}
Department of Health Sciences{break}
University of Leicester, UK{break}

{pstd}
Please report any errors to
{browse "mailto:john.thompson@le.ac.uk":john.thompson@le.ac.uk}.
{p_end}
