*------------------------------
* Code for Chapter 3
* data file: data/polyp.dta
*------------------------------
clear
set scheme sj
set seed 2789102
program drop _all
set more off
pause off
*-------------------------------
* Section 3.2
* Metropolis Algorithm
*-------------------------------
use data/polyp.dta, clear
drop if gender == "Male"

*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
    args logp b

    local lambda = `b'[1,1]
    scalar `logp' = 0
    logdensity poisson `logp' y `lambda' [fw=f]
    scalar `logp' = `logp' - 390*log(1-exp(-`lambda'))
    logdensity gamma `logp' `lambda' 3 0.3
end

*-------------------------------
* code for a single update
*-------------------------------
matrix theta = (0)
logpost logp theta
matrix newtheta = theta[1,1] + 0.5*rnormal()
logpost newlogp newtheta
scalar logA = newlogp - logp
matrix theta[1,1] = cond(log(runiform())<logA,newtheta[1,1],theta[1,1])  

*-------------------------------
* code for 100 updates
*-------------------------------
set obs 100
gen lambda = 0
matrix theta = (0.1)
forvalues i=1/100 {
   logpost logp theta
   matrix newtheta = theta[1,1] + 0.5*rnormal()
   logpost newlogp newtheta
   scalar logA = newlogp - logp
   matrix theta[1,1] = cond(log(runiform())<logA,newtheta[1,1],theta[1,1])  
   display %10.4f theta[1,1] %10.4f logp %10.4f newtheta[1,1] %10.4f newlogp 
   qui replace lambda = theta[1,1] in `i'
}
mcmctrace lambda
pause "Press q to continue"
*================================================================================
*-------------------------------
* Metropolis-Hastings Algorithm
*-------------------------------
program drop _all
use data/polyp.dta, clear
drop if gender == "Male"
*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
    args logp b

    local lambda = `b'[1,1]
    scalar `logp' = 0
    logdensity poisson `logp' y `lambda' [fw=f]
    scalar `logp' = `logp' - 390*log(1-exp(-`lambda'))
    logdensity gamma `logp' `lambda' 3 0.3
end
*-------------------------------
* program for proposals
*-------------------------------
program genprop
    args theta newtheta

    matrix newtheta = theta[1,1] + 0.1*rnormal()
end
*-------------------------------
* program for log-density of proposal
*-------------------------------
program logprop
    args logp b m

    scalar `logp' = 0
	logdensity normal `logp' `b'[1,1] `m'[1,1] 1
end

*-------------------------------
* code for a single update
*-------------------------------
matrix theta = (0)
logpost logp theta
genprop theta newtheta
logpost newlogp newtheta
logprop logf theta newtheta
logprop newlogf newtheta theta
scalar logA = newlogp + logf - logp - newlogf
matrix theta[1,1] = cond(log(runiform())<logA,newtheta[1,1],theta[1,1]) 

*-------------------------------
* code for 100 uodates
*-------------------------------
set obs 100
gen lambda = 0
matrix theta = (0.1)
forvalues i=1/100 {
   logpost logp theta
   genprop theta newtheta
   logpost newlogp newtheta
   logprop logf theta newtheta
   logprop newlogf newtheta theta
   scalar logA = newlogp + logf - logp - newlogf
   matrix theta[1,1] = cond(log(runiform())<logA,newtheta[1,1],theta[1,1]) 
   display %10.4f theta[1,1]
   qui replace lambda = theta[1,1] in `i'
}
mcmctrace lambda
pause "Press q to continue"
*================================================================================
*-------------------------------
* Metropolis-Hastings using mhsnorm
*-------------------------------
program drop _all
use data/polyp.dta, clear
drop if gender == "Male"

*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
    args logp b

    local lambda = `b'[1,1]
    scalar `logp' = 0
    logdensity poisson `logp' y `lambda' [fw=f]
    scalar `logp' = `logp' - 390*log(1-exp(-`lambda'))
    logdensity gamma `logp' `lambda' 3 0.3
end

*-------------------------------
* code for 200 updates
*-------------------------------
tempname pf
postfile `pf' theta using temp/temp.dta, replace 
matrix b = (0.5)
forvalues i=1/200 {
   mhsnorm logpost b 1 , sd(0.1)
   post `pf' (b[1,1])
}
postclose `pf'

use temp/temp.dta, clear
mcmctrace theta
pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 3.4
* Plot gamma prior
* figure 3.1
*-------------------------------
program drop _all
use data/polyp.dta, clear
drop if gender == "Male"
set seed 358271 

*-------------------------------
* plot the prior
*-------------------------------
twoway function y=gammaden(3,0.3,0,x), range(0 3) ///
   xtitle(lambda) yscale(off)
* FIGURE 3.1  
graph export eps/polypprior.eps, replace

pause "Press q to continue"
*================================================================================
*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
    args logp b

    local lambda = `b'[1,1]
    scalar `logp' = 0
    logdensity poisson `logp' y `lambda' [fw=f]
    scalar `logp' = `logp' - 390*log(1-exp(-`lambda'))
    logdensity gamma `logp' `lambda' 3 0.3
end

*-------------------------------
* 200 updates
*-------------------------------
tempname pf
postfile `pf' lambda using temp/temp.dta, replace 
matrix b = (0.5)
local lp = .
forvalues i=1/200 {
   mhslogn logpost b 1 , sd(0.5) logp(`lp')
   local lp = r(logp)
   post `pf' (b[1,1]) 
}
postclose `pf'
use temp/temp.dta, clear
mcmctrace lambda
* FIGURE 3.2  
graph export eps/chap3_1.eps, replace

pause "Press q to continue"
*================================================================================
*-------------------------------
* Section 3.4
* figure 3.3
* Analysis with mcmcrun
*-------------------------------
program drop _all
use data/polyp.dta, clear
drop if gender == "Male"
set seed 2789102

*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
    args logp b

    local lambda = `b'[1,1]
    scalar `logp' = 0
    logdensity poisson `logp' y `lambda' [fw=f]
    scalar `logp' = `logp' - 390*log(1-exp(-`lambda'))
    logdensity gamma `logp' `lambda' 3 0.3
end

*-------------------------------
* call mcmcrun
*-------------------------------
matrix b = (0.5)
mcmcrun logpost b using temp/temp.csv, samp(mhslogn, sd(0.1) ) ///
  replace burn(500) up(10000) param(lambda) dots(0) thin(5) 
import delimited temp/temp.csv, clear
gen i = _n
mcmcstats lambda

mcmctrace lambda if i >1000 & i <1200
graph save temp/plot1.gph , replace
mcmctrace lambda
graph save temp/plot2.gph , replace
graph combine temp/plot1.gph temp/plot2.gph
* FIGURE 3.3  
graph export eps/chap3_23.eps, replace
pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 3.5
* Scaling the proposal sd
*-------------------------------
program drop _all
set seed 2789102
use data/polyp.dta, clear
drop if gender == "Male"

*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
    args logp b

    local lambda = `b'[1,1]
    scalar `logp' = 0
    logdensity poisson `logp' y `lambda' [fw=f]
    scalar `logp' = `logp' - 390*log(1-exp(-`lambda'))
    logdensity gamma `logp' `lambda' 3 0.3
end

*-------------------------------
* adapt the sd
*-------------------------------
local s = 0.5
forvalues i=1/1000 {
    mhslogn logpost b 1 , sd(`s')
    if r(accept) == 0 local s = `s'/1.01
    else local s = `s'*1.01
}

display "tunied proposal sd " %8.3f `s'

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 3.6
* Use of mcmcrun command to adapt
*-------------------------------
program drop _all
set seed 2789102
use data/polyp.dta, clear
drop if gender == "Male"

*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
    args logp b

    local lambda = `b'[1,1]
    scalar `logp' = 0
    logdensity poisson `logp' y `lambda' [fw=f]
    scalar `logp' = `logp' - 390*log(1-exp(-`lambda'))
    logdensity gamma `logp' `lambda' 3 0.3
end

matrix b = (0.5)
matrix s = (0.1)
mcmcrun logpost b using temp/temp.csv, samp(mhslogn, sd(s[1,1]) ) ///
  replace burn(500) up(10000) param(lambda) dots(0) thin(5) adapt
import delimited temp/temp.csv, clear
mcmcstats lambda

use data/polyp.dta, clear
drop if gender == "Male"
matrix b = (0.5)
matrix s = (0.1)
mcmcrun logpost b using temp/temp.csv, samp(mhslogn, sd(s[1,1]) ) replace burn(500) up(10000) param(lambda) dots(0) thin(5) adapt
pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 3.8
* Zero-truncated Generalized Poisson
* figure 3.4
*-------------------------------
program drop _all
use data/polyp.dta, clear
drop if gender == "Male"
set seed 2789102

*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
    args logp b

    local mu    = `b'[1,1]
    local alpha = `b'[1,2]
    local theta = `mu'/(1+`alpha'*`mu')
    tempvar ay lnp
    gen `ay' = 1+ `alpha'*y
    gen `lnp' = y*log(`theta')+(y-1)*log(`ay')-`theta'*`ay'-log(1-exp(-`theta'))
    qui summarize `lnp' [fw=f]
    scalar `logp' = r(sum)
    logdensity gamma `logp' `mu'    3 0.3
    logdensity gamma `logp' `alpha' 0.5 2
end    

*-------------------------------
* run with mcmcrun
*-------------------------------
matrix b = (1, 0.1)
mcmcrun logpost b using temp/temp.csv, ///
   samp( (mhslogn ,sd(0.1)) (mhslogn, sd(0.2)) ) ///
   param(mu alpha) update(2000) replace 
import delimited temp/temp.csv, clear
mcmctrace mu alpha , cgopt(row(2))
* FIGURE 3.4  
graph export eps/GPinitial.eps, replace
* FIGURE 3.5  
scatter mu alpha
graph export eps/GPinitialscatter.eps, replace
count if mu == mu[_n-1]
count if alpha == alpha[_n-1]

pause "Press q to continue"
*================================================================================
*-------------------------------
* Section 3.8
* Zero-truncated Generalized Poisson
* longer run
*-------------------------------

use data/polyp.dta, clear
drop if gender == "Male"
set seed 2789102
program drop _all

*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
    args logp b

    local mu    = `b'[1,1]
    local alpha = `b'[1,2]
    local theta = `mu'/(1+`alpha'*`mu')
    tempvar ay lnp
    gen `ay' = 1+ `alpha'*y
    gen `lnp' = y*log(`theta')+(y-1)*log(`ay')-`theta'*`ay'-log(1-exp(-`theta'))
    qui summarize `lnp' [fw=f]
    scalar `logp' = r(sum)
    logdensity gamma `logp' `mu'    3 0.3
    logdensity gamma `logp' `alpha' 0.5 2
end    

*-------------------------------
* run of 100,000 thin by 5
*-------------------------------
matrix b = (1, 0.1)
mcmcrun logpost b using temp/temp.csv, ///
   samp( (mhslogn ,sd(0.1)) (mhslogn, sd(0.2)) ) ///
   param(mu alpha) update(100000) burn(1000) thin(5) replace 
import delimited temp/temp.csv, clear

mcmcstats mu alpha
pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 3.9.1
* Centering
* figure 3.6 - without centering
*-------------------------------

clear
program drop logpost
*-------------------------------
* Simulate data
*-------------------------------
set seed 826015
set obs 50
gen x = round(40+30*runiform())
gen y = round(90+0.9*x+25*rnormal())
save temp/regdata.dta, replace

*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
	args logp b

	local alpha = `b'[1,1]
	local beta = `b'[1,2]
	local sigma = `b'[1,3]
	scalar `logp' = 0
	logdensity normal `logp' y `alpha'+`beta'*x `sigma'
	logdensity normal `logp' `alpha' 100 100
	logdensity normal `logp' `beta'  1 1
	logdensity uniform `logp' `sigma' 0 50
end	

*-------------------------------
* run with mcmcrun
*-------------------------------
matrix b = (80, 0.8, 25)
matrix s = (15, 0.2, 0.75)
mcmcrun logpost b using temp/temp.csv, samp( 2(norm, sd(s)) (trnc , sd(s) lb(0) ub(50)) ) ///
   replace burn(1000) adapt update(1000) par(alpha beta sigma)
import delimited temp/temp.csv, clear
mcmcstats *
* FIGURE 3.6  
graph matrix alpha beta sigma , msize(small)
graph export eps/chap3_matrix.eps, replace

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 3.9.1
* Centering
* figure 3.6 - with centering
*-------------------------------
program drop _all
use temp/regdata.dta, clear

*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
	args logp b

	local alpha = `b'[1,1]
	local beta = `b'[1,2]
	local sigma = `b'[1,3]
	scalar `logp' = 0
	logdensity normal `logp' y `alpha'+`beta'*(x-55) `sigma'
	logdensity normal `logp' `alpha' 155 100
	logdensity normal `logp' `beta'  1 1
	logdensity uniform `logp' `sigma' 0 50
end

*-------------------------------
* run with mcmcrun
*-------------------------------
matrix b = (155, 0.8, 25)
matrix s = (15, 0.2, 0.75)
mcmcrun logpost b using temp/temp.csv, samp( 2(norm, sd(s)) (trnc , sd(s) lb(0) ub(50)) ) ///
   replace burn(1000) adapt update(1000) par(alpha beta sigma) jpost
import delimited temp/temp.csv, clear
label var alpha "alpha*"
mcmcstats *
* FIGURE 3.7  
graph matrix alpha beta sigma  , msize(small)
graph export eps/chap3_matrix_centred.eps, replace

replace alpha = alpha - 55*beta
mcmcstats *

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 3.9.2
* Block update of regression problem
*-------------------------------

use temp/regdata.dta, clear
program drop _all
*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
	args logp b

	local alpha = `b'[1,1]
	local beta = `b'[1,2]
	local sigma = `b'[1,3]
	scalar `logp' = 0
	logdensity normal `logp' y `alpha'+`beta'*x `sigma'
	logdensity normal `logp' `alpha' 155 100
	logdensity normal `logp' `beta'  1 1
	logdensity uniform `logp' `sigma' 0 50
end	


*-------------------------------
* run with mcmcrun
*-------------------------------
matrix b = (80, 0.8, 25)
matrix V = (400,-7.6\-7.6,0.16)
matrix L = cholesky(V)
mcmcrun logpost b using temp/temp.csv, ///
   samp((mhsmnorm , chol(L) ) (mhstrnc , sd(0.75) lb(0) ub(50))) ///
   par(alpha beta sigma) burn(1000) update(5000) jpost replace
import delimited temp/temp.csv, clear
graph matrix alpha beta sigma 
mcmcstats *

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 3.9.2
* Block update polyps 
*-------------------------------

program drop _all
set seed 2789102
use data/polyp.dta, clear
drop if gender == "Male"
*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
    args logp b

    local mu    = exp(`b'[1,1])
    local alpha = exp(`b'[1,2])
    local theta = `mu'/(1+`alpha'*`mu')
    tempvar ay lnp
    gen `ay' = 1+ `alpha'*y
    gen `lnp' = y*log(`theta')+(y-1)*log(`ay')-`theta'*`ay'-log(1-exp(-`theta'))
    qui summarize `lnp' [fw=f]
    scalar `logp' = r(sum)
    logdensity gamma `logp' `mu'    3 0.3
    logdensity gamma `logp' `alpha' 0.5 2
    scalar `logp' = `logp' + `b'[1,1] + `b'[1,2]
end    

matrix b = ( 1.0 , 0.1 )
matrix V = ( .11, -0.19 \ -0.19, 0.36 )
matrix L = cholesky(V)
mcmcrun logpost b using temp/temp.csv, samp(mhsmnorm, ch(L) ) ///
  par(lmu lalpha) burn(1000) update(100000) thin(5) replace jpost
import delimited temp/temp.csv, clear
mcmcstats *
gen mu = exp(lmu)
gen alpha = exp(lalpha)

mcmcstats mu alpha

pause "Press q to continue"
*================================================================================
