*------------------------------
* Code for Chapter 7
* working directory must include the data file: seeds.dta
*------------------------------
clear
set more off
set scheme sj
program drop _all
pause off

*-------------------------------
* Section 7.2
* simulated logistic in Stata
*-------------------------------
program logpost
   args logp b ipar

   tempvar pr
   gen `pr' = invlogit(`b'[1,1]+`b'[1,2]*x)
   scalar `logp' = 0
   logdensity binomial `logp' y n `pr'
   logdensity normal `logp' `b'[1,1] 0 4
   logdensity normal `logp' `b'[1,2] 0 4
end

set obs 50
set seed 407188
gen n = 1 + floor(runiform()*10)
gen x = rnormal(0,1)
gen p = invlogit(-1 + 0.5*x )
gen y = rbinomial(n,p)
matrix theta = J(1,2,0)
matrix s = J(1,2,0.2)
mcmcrun logpost theta using temp/temp.csv, samp( 2(mhsnorm, sd(s)) ) ///
  burn(1000) adapt update(5000) jpost dots(0) savelogp            ///
  par(beta0 beta1)  replace
import delimited temp/temp.csv, clear
mcmcstats beta*

pause "Press q to continue"
*===================================================================================================
*-------------------------------
* Section 7.2
* simulated logistic in Mata
*-------------------------------
clear
program drop _all
set more off
set matastrict off
mata:
mata clear
real scalar logpost(real matrix X,real rowvector b,real scalar ipar)
{
  p = invlogit(b[1]:+b[2]*X[,3])
  return(logdensity("binomial",X[,1],X[,2],p) +logdensity("normal",b,0,4))
}
end

set seed 407188
set obs 50
gen n = 1 + floor(runiform()*10)
gen x = rnormal(0,1)
gen p = invlogit(-1 + 0.5*x )
gen y = rbinomial(n,p)
matrix theta = J(1,3,0)
matrix s = J(1,3,0.2)
mcmcrun logpost X theta using temp/temp.csv, samp( 2(norm, sd(s)) ) ///
   burn(1000) adapt update(5000) jpost dots(0) savelogp           ///
   par(beta0 beta1)  replace data(X=(y n x) theta s) mata
import delimited temp/temp.csv, clear
mcmcstats beta*

pause "Press q to continue"
*===================================================================================================

*-------------------------------
* Section 7.2
* simulated logistic in Mata
* matasctrict on
*-------------------------------
program drop _all
set more off

set matastrict on
mata:
mata clear
real scalar logpost(real matrix X,real rowvector b,real scalar ipar)
{
  real colvector p 
  
  p = invlogit(b[1]:+b[2]*X[,3])
  return(logdensity("binomial",X[,1],X[,2],p) +logdensity("normal",b,0,4))
}
end

clear
set seed 407188
set obs 50
gen n = 1 + floor(runiform()*10)
gen x = rnormal(0,1)
gen p = invlogit(-1 + 0.5*x )
gen y = rbinomial(n,p)

matrix theta = J(1,3,0)
matrix s = J(1,3,0.2)
mcmcrun logpost X theta using temp/temp.csv, samp( 2(norm, sd(s)) ) ///
burn(1000) adapt update(5000) jpost dots(0) savelogp par(beta0 beta1)  replace ///
data(X=(y n x) theta s) mata

import delimited temp/temp.csv, clear
mcmcstats beta*

pause "Press q to continue"
*===================================================================================================
*-------------------------------
* Section 7.3
* ordinal regression 
* Cook's method
*-------------------------------
program drop _all
mata:
mata clear
mata set matastrict off
real scalar logpost(real matrix X,real rowvector b,real scalar ipar)
{
    alpha = J(4,1,0)
    alpha[1] = b[1]
    alpha[2] = alpha[1] + b[4]
    alpha[3] = alpha[2] + b[5]
    alpha[4] = alpha[3] + b[6]
    beta  = J(3,1,0)
    beta[2]  = b[2]
    beta[3]  = b[3]
    p = J(15,1,0)
    cp = J(4,1,0)
    for (k=1;k<=3;k++) {
       cp = invlogit(alpha:+beta[k])
       p[5*k-4] = cp[1]
       p[5*k-3] = cp[2] - cp[1]
       p[5*k-2] = cp[3] - cp[2]
       p[5*k-1] = cp[4] - cp[3]
       p[5*k]   = 1 - cp[4]
    }
    return(sum(X[,1]:*log(p)) - 0.5*b[1]*b[1] - 2*b[2]*b[2] - 2*b[3]*b[3]
        + 4*log(b[4])-5*b[4] + 4*log(b[5])-5*b[5] + 4*log(b[6])-5*b[6])
}
end
   

tempname rf
postfile `rf' alpha1 q1 m1 sd1 se1 beta2  q2 m2 sd2 se2 ///
              beta3  q3 m3 sd3 se3 delta2 q4 m4 sd4 se4 ///
              delta3 q5 m5 sd5 se5 delta4 q6 m6 sd6 se6 ///
              using temp/ordinal_valid.dta,replace
set seed 639125
forvalues rep=1/20 {
   qui drop _all
   qui set obs 174
   gen trt = 1 + (_n>66) + (_n>119)
   gen u = runiform()
   gen appetite = 1
   local alpha1 = rnormal(0,1)
   local beta2  = rnormal(0,0.5)
   local beta3  = rnormal(0,0.5)
   local delta2 = rgamma(5,0.2)
   local delta3 = rgamma(5,0.2)
   local delta4 = rgamma(5,0.2)
   local alpha2 = `alpha1' + `delta2'
   local alpha3 = `alpha2' + `delta3'
   local alpha4 = `alpha3' + `delta4'
   local beta1 = 0
   forvalues k=1/3 {
      forvalues j=1/4 {
         local cp`j' = invlogit(`alpha`j''+`beta`k'')
         qui replace appetite = appetite + 1 if u > `cp`j'' & trt == `k' 
      }
   }
   matrix X = J(15,1,0)
   forvalues i=1/3 {
      forvalues j=1/5 {
         qui count if trt == `i' & appetite == `j'        
         matrix X[5*`i'- 5  + `j',1] = r(N)
      }
   }
   matrix list X
   matrix b = (`alpha1',`beta2',`beta3',`delta2',`delta3',`delta4')
   matrix s = J(1,6,0.5)
   mcmcrun logpost X b using temp/temp.csv,               ///
      samp( 3(mhsnorm, sd(s)) 3(mhslogn, sd(s)) )  ///
      burnin(1000) adapt updates(20000) thin(4)      ///
      replace par( alpha1 beta2 beta3 delta2-delta4) ///
      dots(0) mata data(X s b) jpost 
   import delimited temp/temp.csv, clear
   local n = _N
   local postit ""
   foreach v of varlist a* b* d* {
      qui count if `v' < ``v''
      local q`v' = (r(N)+0.5)/(`n'+1)
      qui mcmcstats `v'
      local postit "`postit' (``v'') (`q`v'') (`r(mn1)') (`r(sd1)') (`r(se1)')"
   }
   post `rf' `postit'
}
postclose `rf'

use temp/ordinal_valid.dta, clear
local i = 0
foreach v of varlist q* {
   local ++i
   gen z`i' = invnormal(q`i')
   gen c2`i' = z`i'*z`i'
   qui summarize c2`i'
   display %6s "`v'" %8.3f `r(sum)' %8.4f chi2tail(_N,`r(sum)')
}

pause "Press q to continue"
*===================================================================================================
*-------------------------------
* Section 7.4
* broomrape basic analysis 
* figure 7.1
*-------------------------------
use data/broomrape.dta, clear
summarize seed 
local m1 = r(mean)
gen x1 = seed - `m1'
summarize root 
local m2 = r(mean)
gen x2 = root - `m2'
gen x12 = root*seed
summarize x12 
local m12 = r(mean)
replace x12 = x12 - `m12'
set matastrict off
mata:
mata clear
real scalar logpost(real matrix X,real rowvector b,real scalar ipar)
{
  sd = 1/sqrt(b[5])
  eta = b[1]:+b[2]*X[,3]+b[3]*X[,4]+b[4]*X[,5]+b[|6\26|]'
  p = invlogit(eta)
  return(logdensity("binomial",X[,1],X[,2],p) 
       + logdensity("normal",b[|6\26|]',0,sd) 
       + logdensity("normal",b[|1\4|],0,2) + logdensity("gamma",b[5],2,2) )
}
end

matrix theta = J(1,26,0)
matrix theta[1,5] = 1
mcmcrun logpost X theta using temp/temp.csv, samp( 4(mhsnorm, sd(0.2)) ///
   (mhslogn, sd(0.2)) 21(mhsnorm, sd(0.2)) ) replace burn(1000)   ///
   update(10000) jpost  dots(0) par(a0 a1 a2 a12 tau u1-u21)        ///
   mata data(X=(y n x1 x2 x12) theta)
import delimited temp/temp.csv, clear
replace a0 = a0 - `m1'*a1 - `m2'*a2 - `m12'*a12
gen sd = 1/sqrt(tau)
mcmcstats a* sd 

* FIGURE 4.1  
mcmctrace a1 a2 a12 sd
graph export eps\chap7_mixing.eps, replace

pause "Press q to continue"
*===================================================================================================
*-------------------------------
* Section 7.4
* broomrape basic analysis - adapted
* figure 7.2
*-------------------------------

use data\broomrape.dta, clear
summarize seed 
local m1 = r(mean)
gen x1 = seed - `m1'
summarize root 
local m2 = r(mean)
gen x2 = root - `m2'
gen x12 = root*seed
summarize x12 
local m12 = r(mean)
replace x12 = x12 - `m12'

set matastrict off
mata:

mata clear

real scalar logpost(real matrix X,real rowvector b,real scalar ipar)
{
  sd = 1/sqrt(b[5])
  eta = b[1]:+b[2]*X[,3]+b[3]*X[,4]+b[4]*X[,5]+b[|6\26|]'
  p = invlogit(eta)
  return(logdensity("binomial",X[,1],X[,2],p) + logdensity("normal",b[|6\26|]',0,sd) +
     logdensity("normal",b[|1\4|],0,2) + logdensity("gamma",b[5],2,2) )
}
end

matrix theta = J(1,26,0)
matrix theta[1,5] = 1
matrix s = J(1,26,0.2)
mcmcrun logpost X theta using temp/temp.csv, samp( 4(mhsnorm, sd(s)) ///
   (mhslogn, sd(s)) 21(mhsnorm, sd(s)) ) replace burn(1000)     ///
   update(10000) jpost  dots(0) par(a0 a1 a2 a12 tau u1-u21)      ///
   adapt mata data(X=(y n x1 x2 x12) theta s)


import delimited temp/temp.csv, clear
replace a0 = a0 - `m1'*a1 - `m2'*a2 - `m12'*a12
gen sd = 1/sqrt(tau)
mcmcstats a* sd 

* FIGURE 4.2
mcmctrace a1 a2 a12 sd
graph export  eps\chap7_mixing_tuned.eps, replace

pause "Press q to continue"
*===================================================================================================
*-------------------------------
* Section 7.4
* broomrape basic analysis - adapted - conditional
*-------------------------------

use data\broomrape.dta, clear
summarize seed 
local m1 = r(mean)
gen x1 = seed - `m1'
summarize root 
local m2 = r(mean)
gen x2 = root - `m2'
gen x12 = root*seed
summarize x12 
local m12 = r(mean)
replace x12 = x12 - `m12'

set matastrict off
mata:
mata clear
real scalar logpost(real matrix X,real rowvector b,real scalar ipar)
{
  sd = 1/sqrt(b[5])
  if( ipar < 5 ) {  
     mu = b[1]:+b[2]*X[,3]+b[3]*X[,4]+b[4]*X[,5]+b[|6\26|]'
     p = invlogit(mu)
     return(logdensity("binomial",X[,1],X[,2],p) 
          + logdensity("normal",b[ipar],0,4) )
  }
  else if( ipar == 5 ) {
     return(logdensity("normal",b[|6\26|],0,sd) 
          + logdensity("gamma",b[5],2,2) )
  }
  else {
     i=ipar-5
     mui = b[1]+b[2]*X[i,3]+b[3]*X[i,4]+b[4]*X[i,5]+b[ipar]
     pi = invlogit(mui)
     return(logdensity("binomial",X[i,1],X[i,2],pi)
          + logdensity("normal",b[ipar],0,sd) )
  }
}
end

matrix theta = J(1,26,0)
matrix theta[1,5] = 1
matrix s = J(1,26,0.2)

mcmcrun logpost X theta using temp/temp.csv, samp( 4(mhsnorm, sd(s)) ///
   (mhslogn, sd(s)) 21(mhsnorm, sd(s)) ) ///
   replace burn(1000) update(10000) dots(0) ///
   par(a0 a1 a2 a12 tau u1-u21) adapt ///
   mata data(X=(y n x1 x2 x12) theta s)


import delimited temp/temp.csv, clear
replace a0 = a0 - `m1'*a1 - `m2'*a2 - `m12'*a12
gen sd = 1/sqrt(tau)
mcmcstats a* sd 
mcmctrace a1 a2 a12 sd

pause "Press q to continue"
*===================================================================================================
*-------------------------------
* Section 7.4
* broomrape basic analysis - adapted - efficient conditional
*-------------------------------

use data\broomrape.dta, clear
summarize seed 
local m1 = r(mean)
gen x1 = seed - `m1'
summarize root 
local m2 = r(mean)
gen x2 = root - `m2'
gen x12 = root*seed
summarize x12 
local m12 = r(mean)
replace x12 = x12 - `m12'

set matastrict off
mata:
mata clear
real scalar logpost(real matrix X,real rowvector b,real scalar ipar)
{
  sd = 1/sqrt(b[5])
  if (ipar < 5) {  
     mu = b[1]:+b[2]*X[,3]+b[3]*X[,4]+b[4]*X[,5]+b[|6\26|]'
     p = invlogit(mu)
     return(sum(X[,1]:*mu:-X[,2]:*log(1:+exp(mu))) - b[ipar]*b[ipar]/32)
  }
  else if( ipar == 5 ) {
     return(-0.5*b[5]*b[|6\26|]*b[|6\26|]' -21*log(sd) + log(b[5]) -b[5]/2)
  }
  else {
     i=ipar-5
     mui = b[1]+b[2]*X[i,3]+b[3]*X[i,4]+b[4]*X[i,5]+b[ipar]
     return(X[i,1]*mui-X[i,2]*log(1+exp(mui)) -0.5*b[5]*b[ipar]*b[ipar])
  }
}
end

matrix theta = J(1,26,0)
matrix theta[1,5] = 1
matrix s = J(1,26,0.2)

mcmcrun logpost X theta using temp/temp.csv, samp( 4(mhsnorm, sd(s)) ///
   (mhslogn, sd(s)) 21(mhsnorm, sd(s)) ) ///
   replace burn(1000) update(10000) dots(0) ///
   par(a0 a1 a2 a12 tau u1-u21) adapt ///
   mata data(X=(y n x1 x2 x12) theta s)


import delimited temp/temp.csv, clear
replace a0 = a0 - `m1'*a1 - `m2'*a2 - `m12'*a12
gen sd = 1/sqrt(tau)
mcmcstats a* sd 
mcmctrace a1 a2 a12 sd

pause "Press q to continue"
*===================================================================================================
*-------------------------------
* Section 7.4
* broomrape basic analysis - adapted - efficient conditional hierarchical
*-------------------------------

use data\broomrape.dta, clear
summarize seed 
local m1 = r(mean)
gen x1 = seed - `m1'
summarize root 
local m2 = r(mean)
gen x2 = root - `m2'
gen x12 = root*seed
summarize x12 
local m12 = r(mean)
replace x12 = x12 - `m12'

set matastrict off
mata:
mata clear
real scalar logpost(real matrix X,real rowvector b,real scalar ipar)
{
  sd = 1/sqrt(b[5])
  if (ipar < 5) {  
     mu = b[1]:+b[2]*X[,3]+b[3]*X[,4]+b[4]*X[,5]
     v = b[|6\26|]'
     return(-0.5*b[5]*sum((v:-mu):*(v:-mu)) - b[ipar]*b[ipar]/32)
  }
  else if (ipar == 5) {
     mu = b[1]:+b[2]*X[,3]+b[3]*X[,4]+b[4]*X[,5]
     v = b[|6\26|]'
     return(-0.5*b[5]*sum((v:-mu):*(v:-mu)) - 21*log(sd) + log(b[5]) - b[5]/2)
  }
  else {
     i=ipar-5
     mui = b[1]+b[2]*X[i,3]+b[3]*X[i,4]+b[4]*X[i,5]
     return(X[i,1]*b[ipar]-X[i,2]*log(1+exp(b[ipar]))
            - 0.5*b[5]*(b[ipar]-mui)*(b[ipar]-mui) )
  }
}
end

matrix theta = J(1,26,0)
matrix theta[1,5] = 1
matrix s = J(1,26,0.2)

mcmcrun logpost X theta using temp/temp.csv, samp( 4(mhsnorm, sd(s)) ///
   (mhslogn, sd(s)) 21(mhsnorm, sd(s)) ) ///
   replace burn(1000) update(10000) dots(0) ///
   par(a0 a1 a2 a12 tau u1-u21) adapt ///
   mata data(X=(y n x1 x2 x12) theta s)

import delimited temp/temp.csv, clear
replace a0 = a0 - `m1'*a1 - `m2'*a2 - `m12'*a12
gen sd = 1/sqrt(tau)
mcmcstats a* sd 
mcmctrace a1 a2 a12 sd

pause "Press q to continue"
*===================================================================================================
*-------------------------------
* Section 7.4
* broomrape basic analysis - adapted - efficient conditional hierarchical + Gibbs
*-------------------------------
program drop _all
use data\broomrape.dta, clear
summarize seed 
local m1 = r(mean)
gen x1 = seed - `m1'
summarize root 
local m2 = r(mean)
gen x2 = root - `m2'
gen x12 = root*seed
summarize x12 
local m12 = r(mean)
replace x12 = x12 - `m12'

set matastrict off
mata:
mata clear
real scalar logpost(real matrix X,real rowvector b,real scalar ipar)
{
  sd = 1/sqrt(b[5])
  if (ipar < 5) {  
     mu = b[1]:+b[2]*X[,3]+b[3]*X[,4]+b[4]*X[,5]
     v = b[|6\26|]'
     return( -0.5*b[5]*sum((v:-mu):*(v:-mu)) - b[ipar]*b[ipar]/32 )
  }
  else if (ipar == 5) {
     mu = b[1]:+b[2]*X[,3]+b[3]*X[,4]+b[4]*X[,5]
     v = b[|6\26|]'
     return( -0.5*b[5]*sum((v:-mu):*(v:-mu)) - 21*log(sd) + log(b[5]) - b[5]/2  )
  }
  else {
     i=ipar-5
     mui = b[1]+b[2]*X[i,3]+b[3]*X[i,4]+b[4]*X[i,5]
        return(X[i,1]*b[ipar]-X[i,2]*log(1+exp(b[ipar]))
           - 0.5*b[5]*(b[ipar]-mui)*(b[ipar]-mui) )
  }
}
void myGibbs(real scalar logp,pointer(function) userprog,
             real matrix X, real rowvector b,real scalar ipar)
{
   mu = b[1]:+b[2]:*X[,3]:+b[3]:*X[,4]:+b[4]:*X[,5]
   r = b[|6\26|]'-mu
   b[5] = rgamma(1,1,12.5,2/(1+r'*r) )
}
end
matrix theta = J(1,26,0)
matrix theta[1,5] = 1
matrix s = J(1,26,0.2)
mcmcrun logpost X theta using temp/temp.csv, samp( 4(mhsnorm, sd(s)) ///
   (myGibbs) 21(mhsnorm, sd(s)) ) replace burn(1000)           ///
   update(10000) dots(0) par(a0 a1 a2 a12 tau u1-u21) adapt       ///
   mata data(X=(y n x1 x2 x12) theta s)

import delimited temp/temp.csv, clear
replace a0 = a0 - `m1'*a1 - `m2'*a2 - `m12'*a12
gen sd = 1/sqrt(tau)
mcmcstats a* sd 
mcmctrace a1 a2 a12 sd

pause "Press q to continue"
*===================================================================================================
*---------------------------------
* Adding to logdensity - not intended to run
*---------------------------------
capture noisily {
mata:
real scalar logdensity(
    string scalar distribution,
    real matrix y,
    real matrix p1,
  | real matrix p2,
    real matrix p3,
    real matrix p4)
{
    if (distribution == "bernoulli") {
        ...
    }
        ...
    else if (distribution == "gamma") {
        if (args() < 4 || args() > 5) exit(3000)
        if (args() == 5) w = p3; else w = 1
        return( sum(w:*((p1:-1):*log(y):-y:/p2:-p1:*log(p2):-lngamma(p1))) )
    }
        ...
}
mata mlib add libwb logdensity()
end

} // end capture block
